
/*	File			BullyElectionAlgorithmDlg.cpp
	Purpose			Implementation of the Bully Election Algorithm
	Author			Richard Anthony	(R.J.Anthony@gre.ac.uk)
	Date			January 2015

	Demonstration implementation of the Bully election algorithm.
	Normal process states in this algorithm are State_Slave and State_Master.
	
	All processes are initialised in State_Slave. In this state they listen for MT_MasterHeartbeat messages,
	sent periodically by a master (if one exists). If one or more slaves miss a sequence of the heartbeat 
	messages (indicating absence of a master) they initiate an election by sending a MT_ElectionTrigger message.

	The process that wins the election will send MT_MasterHeartbeat messages periodically. A very important
	rule is that there must only be one State_Master instance present at any time, and therefore the election
	mechanism must be robust to prevent multiple masters locally determining that they have won an election.
	To facilitate this a process initially enters the State_Candidate state during an election. This is a 
	transient state used to ensure that all challenges have been resolved before a single process elevates to
	State_Master.

	The following design assumptions are made:
	1.	Elections will occur between processes in a local network and therefore broadcast communication can be used.
	2.	Only one participating process will be present on each computer, so the IP address of the host computer can be
		used as the process ID for the purposes of the election (this is a simple way to ensure that IDs are unique).
	3.	The process with the highest ID (its IP address) wins an election.
	4.	Once a single master exists, slaves with higher IDs may join the system (without triggering an election).
*/

#include "stdafx.h"
#include "BullyElectionAlgorithm.h"
#include "BullyElectionAlgorithmDlg.h"
#include "afxdialogex.h"
#pragma warning(disable : 4996)

IMPLEMENT_DYNAMIC(CBullyElectionAlgorithmDlg, CDialog)
CBullyElectionAlgorithmDlg::CBullyElectionAlgorithmDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBullyElectionAlgorithmDlg::IDD, pParent)
{	
	m_LocalState = State_Slave;
	m_iMasterHeartbeatClock = 0;	// Used to measure time intervals between heartbeats from the Master instance
	m_iMostRecentHeartbeat_TimeDetected = m_iMasterHeartbeatClock;
	m_iClock_CountInCandidateState_SincelastChallenge = 0;

	bool bResult = CreateSendSocket(); // The socket used for all sends
	if(false == bResult)
	{
		OnBnClickedDone();
	}

	bResult = CreateReceiveSocket();	// The socket on which all messages are received
	if(false == bResult)
	{
		OnBnClickedDone();
	}
}

CBullyElectionAlgorithmDlg::~CBullyElectionAlgorithmDlg()
{	
	OnBnClickedDone();
}

void CBullyElectionAlgorithmDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	m_pEventList = (CListBox*) GetDlgItem(IDC_Event_LIST);
	m_pCurrentState = (CEdit*) GetDlgItem(IDC_CurrentState_EDIT);

	InitialiseControls();
	InitiateMasterHeartbeatTimer();
	InitiateMessageReceiveTimer();
	WriteEventListEntry("Initialised (Slave state)");
}

BEGIN_MESSAGE_MAP(CBullyElectionAlgorithmDlg, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_ClearEventList_BUTTON, OnBnClickedCleareventlistButton)
END_MESSAGE_MAP()

void CBullyElectionAlgorithmDlg::InitialiseControls()
{	
	DisplayLocalState();
}

bool CBullyElectionAlgorithmDlg::CreateSendSocket()
{   // UDP, non-blocking, broadcast
	m_SendSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_SendSock)
	{
		return false;	// Error creating socket
	}

	// Turn on Broadcast mode
	char cOpt[2];
	cOpt[0] = 1; // true
	cOpt[1] = 0; // null terminate the option array
	int iError = setsockopt(m_SendSock, SOL_SOCKET, SO_BROADCAST, cOpt, sizeof(cOpt)); 
  	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting broadcast option
	}

	SetUpSendAddressStructForBroadcast(); // All election algorithm messages will be broadcast
	return true;
}

void CBullyElectionAlgorithmDlg::SetUpSendAddressStructForBroadcast()
{	
	m_SendAddr.sin_addr = GetBroadcastAddress();
	m_SendAddr.sin_family = AF_INET;
	m_SendAddr.sin_port = htons(BULLY_ELECTION_ALGORITHM_PORT);
}

IN_ADDR CBullyElectionAlgorithmDlg::GetBroadcastAddress()
{	
	IN_ADDR addr;
	addr.S_un.S_un_b.s_b1 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b2 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b3 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b4 = (unsigned  char) 255;
	return addr;
}

bool CBullyElectionAlgorithmDlg::CreateReceiveSocket()
{   // UDP, non-blocking, broadcast
	m_RecvSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_RecvSock)
	{
		return false;	// Error creating socket
	}

	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(m_RecvSock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting non-blocking behaviour
	}

	m_LocalAddr.sin_addr = GetLocalHostAddressDetails();
	m_LocalAddr.sin_family = AF_INET;
	m_LocalAddr.sin_port = htons((u_short) BULLY_ELECTION_ALGORITHM_PORT);

	int iResult = bind(m_RecvSock, (const SOCKADDR FAR*)&m_LocalAddr, sizeof(m_LocalAddr));
	if(SOCKET_ERROR == iResult)
	{
		MessageBox("Bind failed !\nPort is probably in use","BULLY_ELECTION_ALGORITHM");
		return false;
	}
	return true;
}

IN_ADDR CBullyElectionAlgorithmDlg::GetLocalHostAddressDetails()
{	
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		MessageBox("Failed to get local address!","BULLY_ELECTION_ALGORITHM");
		this->OnCancel();
	}
	return addr;
}

void CBullyElectionAlgorithmDlg::InitiateMasterHeartbeatTimer()
{	
	m_nTimer = SetTimer(TIMER_MASTER_HEARTBEAT_TX, MASTER_HEARTBEAT_TX_PERIOD, 0);
}

void CBullyElectionAlgorithmDlg::InitiateMessageReceiveTimer()
{	
	m_nTimer = SetTimer(TIMER_CHECK_FOR_MESSAGE, CHECK_FOR_MESSAGE_PERIOD, 0);
}

void CBullyElectionAlgorithmDlg::OnTimer(UINT nIDEvent) 
{	
	if(nIDEvent == TIMER_MASTER_HEARTBEAT_TX)	// Master Heartbeat transmission interval (but comes here regardless of state)
	{
		m_iMasterHeartbeatClock++;		// Used to measure time intervals between heartbeats from the Master (only relevent when in Slave state)
		m_iMasterHeartbeatClock %= HeartbeatClock_LIMIT;	// m_iHeartbeatClock counts from 0 to 999, then is reset to 0 and start counting again
		
		switch(m_LocalState) {
			case State_Master:
				SendMasterHeartbeatMessage();	// Only performed if current state is State_Master	
				break;
			case State_Candidate:
				m_iClock_CountInCandidateState_SincelastChallenge++; // Increment Candidate duration clock
				if(CANDIDATE_STATE_DURATION <= m_iClock_CountInCandidateState_SincelastChallenge)
				{	// Have stayed in candidate state for the requisite period (no successful challengers, so now promote to Master)				
					TransitionToMasterState();
				}
				break;
			case State_Slave:
				CheckPresenceOfMaster(m_iMasterHeartbeatClock);		// Check master is still present based on interval since last heartbeat received
				break;
		}
	}
	if(nIDEvent == TIMER_CHECK_FOR_MESSAGE) // Check for receved messages
	{
		DoReceive();
	}
}

void CBullyElectionAlgorithmDlg::CheckPresenceOfMaster(int iCurrentHeartbeatClock)
{	// Check interval since last heartbeat to determine if a Master instance is still present	
	int iEffective_CurrentHeartbeatClock; // Takes care of clock rollovers
	
	// Heartbeat count increments every MASTER_HEARTBEAT_TX_PERIOD milliseconds, up to a maximum of HeartbeatClock_LIMIT increments (modulo)
	// Heartbeat count rolls over when it reaches HeartbeatClock_LIMIT, back to 0 (%HeartbeatClock_LIMIT)
	if(iCurrentHeartbeatClock < m_iMostRecentHeartbeat_TimeDetected)
	{	// Current clock value is lower, so it has rolled over! Treat it as having a value of +HeartbeatClock_LIMIT
		// for the purposes of the calculations (so the current clock value is always higher than the stored value
		// in iMostRecentHeartbeat_TimeDetected).
		iEffective_CurrentHeartbeatClock = iCurrentHeartbeatClock + HeartbeatClock_LIMIT;
	}
	else
	{	// Use Current clock value as is.
		iEffective_CurrentHeartbeatClock = iCurrentHeartbeatClock;
	}

	if((m_iMostRecentHeartbeat_TimeDetected + ELECTION_TRIGGER_HEARTBEATS_MISSED)
		<= iEffective_CurrentHeartbeatClock)
	{	// m_iMostRecentHeartbeat_TimeDetected value is out-of-date, Master is considered to have departed
		TransitionToCandidateState(); //Start election, with self as a candidate
	}
}

void CBullyElectionAlgorithmDlg::TransitionToMasterState()
{
	ChangeState(State_Master);
	SendMasterHeartbeatMessage();	// Immediately send a heartbeat message to prevent further slave elections
}

void CBullyElectionAlgorithmDlg::TransitionToCandidateState()
{
	ChangeState(State_Candidate);
	m_iClock_CountInCandidateState_SincelastChallenge = 0; // Reset Candidate duration clock
	SendElectionTriggerMessage();	//Start election, with self as a candidate
}

void CBullyElectionAlgorithmDlg::TransitionToSlaveState()
{
	ChangeState(State_Slave);
	m_iMostRecentHeartbeat_TimeDetected = m_iMasterHeartbeatClock; 	// Update MostRecentHeartbeat timestamp
}

void CBullyElectionAlgorithmDlg::Send(BULLY_ELECTION_ALGORITHM_Message_Format Message)
{
	int iError;
	int iBytesSent = sendto(m_SendSock, (char *) &Message, sizeof(Message), 0, 
							(const sockaddr *)&m_SendAddr, sizeof(m_SendAddr));
	if(SOCKET_ERROR == iBytesSent)
	{
		iError = WSAGetLastError();
		MessageBox("Failed to send","BULLY_ELECTION_ALGORITHM");
	}
}
				
void CBullyElectionAlgorithmDlg::SendMasterHeartbeatMessage()
{
	BULLY_ELECTION_ALGORITHM_Message_Format Message;
	Message.MessageType = MT_MasterHeartbeat;
	Send(Message);
	WriteEventListEntry("MasterHeartbeatMessage sent");
}

void CBullyElectionAlgorithmDlg::SendElectionTriggerMessage()
{
	BULLY_ELECTION_ALGORITHM_Message_Format Message;
	Message.MessageType = MT_ElectionTrigger;
	Send(Message);
	WriteEventListEntry("ElectionTriggerMessage sent");
}

void CBullyElectionAlgorithmDlg::DoReceive()
{		
	BULLY_ELECTION_ALGORITHM_Message_Format Message;
	SOCKADDR_IN FromAddr; // The address of the sender of the message
	int iSizeOfFromAddress;
	int iBytesRecd = recvfrom(m_RecvSock, (char *) &Message, sizeof(Message), 0, (sockaddr*) &FromAddr, &iSizeOfFromAddress);
	if(SOCKET_ERROR == iBytesRecd)
	{
		int iError = WSAGetLastError();
		if(iError != WSAEWOULDBLOCK)
		{	// If no packet is queued for the socket the operation would block.
			return;
		}
	}
	else
	{
		if(iBytesRecd == 0)
		{
			return;
		}

		if(m_LocalAddr.sin_addr.S_un.S_addr == FromAddr.sin_addr.S_un.S_addr)
		{	// Ignore messages from self (very important when using broadcast)
			WriteEventListEntry("Ignore message from self");
			return;
		}
		
		switch(Message.MessageType)
		{
		  case MT_MasterHeartbeat:
			WriteEventListEntry("MasterHeartbeatMessage received");
			switch(m_LocalState) {
				case State_Master: // Another Master exists (the one with lowest IP address should demote to Slave state)
					WriteEventListEntry("M recd M_Heartbeat");
					CompareNodeIDs_DemoteIfRemoteIsHigher(FromAddr);
					break;
				case State_Candidate: // A Master exists (abandon election and demote to Slave state)
					WriteEventListEntry("C recd M_Heartbeat");
					TransitionToSlaveState();
					break;
				case State_Slave: // A Master exists (no action needed)
					WriteEventListEntry("S recd M_Heartbeat");
					m_iMostRecentHeartbeat_TimeDetected = m_iMasterHeartbeatClock; 	// Update MostRecentHeartbeat timestamp
					break;
			}
			break;
		  case MT_ElectionTrigger:
			WriteEventListEntry("ElectionTriggerMessage received");
			switch(m_LocalState) {
				case State_Master: // Another node thinks the Master does not exist - send heartbeat immediately
					WriteEventListEntry("M recd Election");
					SendMasterHeartbeatMessage();
					break;
				case State_Candidate: // This is the 'election proper' (the candidate with lowest IP address should demote to Slave state)
					WriteEventListEntry("C recd Election");
					CompareNodeIDs_DemoteIfRemoteIsHigher(FromAddr);
					if(State_Candidate == m_LocalState)
					{	// Still in Candidate state, so won the challenge
						SendElectionTriggerMessage(); // Signal the local node's ID to other nodes (do this every time we are challenged)
						m_iClock_CountInCandidateState_SincelastChallenge = 0; // Reset Candidate duration clock
						WriteEventListEntry("Local won");
					}
					break;
				case State_Slave:	// This is the 'election proper' (if the local ID is higher than the sender's then we join the election)
					WriteEventListEntry("S recd Election");
					CompareNodeIDs_PromoteToCandidateIfRemoteIsLower(FromAddr);
					if(State_Candidate == m_LocalState)
					{	// Now in Candidate state, so won the challenge
						WriteEventListEntry("Local won");
					}
					break;
			}
			break;
		}
	}
}


void CBullyElectionAlgorithmDlg::CompareNodeIDs_DemoteIfRemoteIsHigher(SOCKADDR_IN FromAddr)
{
	if(false == CompareNodeIDs_RemoteIsLower(FromAddr))
	{	// Remote has higher address - the local node looses and must demote to Slave state
		TransitionToSlaveState();
	}
}

void CBullyElectionAlgorithmDlg::CompareNodeIDs_PromoteToCandidateIfRemoteIsLower(SOCKADDR_IN FromAddr)
{
	if(true == CompareNodeIDs_RemoteIsLower(FromAddr))
	{	// Remote has lower address - the local node wins and must promote to Candidate state
		TransitionToCandidateState();
	}
}

bool CBullyElectionAlgorithmDlg::CompareNodeIDs_RemoteIsLower(SOCKADDR_IN FromAddr)
{
	if(m_LocalAddr.sin_addr.S_un.S_addr > FromAddr.sin_addr.S_un.S_addr)
	{	// Local node has the higher ID (its IP address)
		return true;
	}
	return false; // Local node has the lower ID (its IP address)
}

void CBullyElectionAlgorithmDlg::ChangeState(BULLY_ELECTION_ALGORITHM_Process_State NewState)
{
	CString sPrevious = ConvertLocalStateToString(); // Remember previous state for display update
	m_LocalState = NewState;								// Do the state update
	CString sNew = ConvertLocalStateToString();	
	CString sDisplay;
	sDisplay.Format("StateChange %s -> %s",sPrevious,sNew);
	WriteEventListEntry(sDisplay);
	DisplayLocalState();
}

void CBullyElectionAlgorithmDlg::DisplayLocalState()
{
	CString sState = ConvertLocalStateToString();
	m_pCurrentState->SetWindowText(sState);
}

CString CBullyElectionAlgorithmDlg::ConvertLocalStateToString()
{			
	switch(m_LocalState) {
		case State_Master:
			return "Master";
			break;
		case State_Candidate:
			return "Candidate";
			break;
		case State_Slave:
			return "Slave";
			break;
	}
	return "Unknown";
}

void CBullyElectionAlgorithmDlg::WriteEventListEntry(CString csLine)
{
	m_pEventList->AddString(csLine);
}

void CBullyElectionAlgorithmDlg::OnBnClickedCleareventlistButton()
{
	while(0 < m_pEventList->GetCount())
	{
		m_pEventList->DeleteString(0);
	}
}

void CBullyElectionAlgorithmDlg::OnBnClickedDone()
{
	closesocket(m_SendSock);
	closesocket(m_RecvSock);
}

