/*	File			CaseStudyGame_Server_DSDlg.cpp
	Purpose			Network Game (Noughts and Crosses) SERVER
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			January 2015

	This version uses the Directory Service:
		The server (this) registers with the Directory Service,
		Clients use the Directory service to locate the Server.


	Documentation for the extension of the original game, to use the Directory Service
	**********************************************************************************
	The interface to the directory service is defined as follows:

	Protocol used by the Directory Service: UDP

	Port number allocations
		From_DIRECTORY_SERVICE_PORT	8002	
		The port the application client must listen on for DirectoryServiceReply messages sent in response to RESOLVE request messages.
	
		To_DIRECTORY_SERVICE_PORT	8008
		The port that application clients use to send RESOLVE messages to the directory service (broadcast is used).
		The port that application servers use to send REGISTER messages to the directory service (broadcast is used).

	Message structures
		The message structure returned by the directory service in response to RESOLVE request messages is shown below:
 
		struct DirectoryServiceReply {
			// IP address followed by port
			unsigned char a1;
			unsigned char a2;
			unsigned char a3;
			unsigned char a4;
			PORT port;
			};

	Message formats
		Client RESOLVE request messages are encoded as a character string containing RESOLVE:server_name where server_name 
		can be a maximum of 30 characters.
		An example is      	RESOLVE:GameServer

		Server REGISTER messages are encoded as a character string containing "REGISTER:server_name:port" where server_name 
		can be a maximum of 30 characters, and the port number is expressed in text form.
		An example is 	REGISTER:GameServer:8004

		Note that for both message types the IP address of the sending process is extracted by the directory service, directly
		from the header of the received message; the game component does not have to explicitly send it.
*/

#include "stdafx.h"
#include "CaseStudyGame_Server_DS.h"
#include "CaseStudyGame_Server_DSDlg.h"
#include <sys/types.h>
#include <sys/timeb.h>
#pragma warning(disable : 4996)

IMPLEMENT_DYNAMIC(CCaseStudyGame_Server_DSDlg, CDialog)
CCaseStudyGame_Server_DSDlg::CCaseStudyGame_Server_DSDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCaseStudyGame_Server_DSDlg::IDD, pParent)
{	
	SeedRandomNumberGenerator();
	
	m_csAppName = GAME_SERVER_NAME;
	bRegisterAlreadyDone = false;

	bool bResult = CreateRegisterSocket();
	if(false == bResult)
	{
		OnBnClickedClose();
	}
	
	SetUpSendAddressStructForBroadcast(m_RegisterAddr, To_DIRECTORY_SERVICE_PORT);

	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		m_ConnectionArray[i].bInUse = false;
	}	
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		m_GameArray[i].bInUse = false;
	}
}

CCaseStudyGame_Server_DSDlg::~CCaseStudyGame_Server_DSDlg()
{	
	CloseAll();
}

void CCaseStudyGame_Server_DSDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	// controls initialisation
	m_pLocalAddress = (CEdit*) GetDlgItem(IDC_ADDRESS);
	m_pListenPort = (CEdit*) GetDlgItem(IDC_PORT);
	m_pEnableServer = (CButton*) GetDlgItem(IDC_ENABLE_SERVER);
	m_pStatus = (CEdit*) GetDlgItem(IDC_STATUS);
	DDX_Control(pDX, IDC_ACTIVE_GAMES_COUNT, m_ActiveGamesCount);
	DDX_Control(pDX, IDC_CONNECTED_COUNT, m_ConnectedCount);

	CString csStr;
	InitialiseLocalAddressStructAndPort();	
	InitialiseControls();
}

BEGIN_MESSAGE_MAP(CCaseStudyGame_Server_DSDlg, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_ENABLE_SERVER, OnBnClickedEnableServer)
	ON_BN_CLICKED(ID_CLOSE, OnBnClickedClose)
	ON_BN_CLICKED(ID_CLEAR_EVENT_LIST, OnBnClickedClearEventList)
END_MESSAGE_MAP()

void CCaseStudyGame_Server_DSDlg::InitialiseControls()
{	
	m_pListenPort->EnableWindow(true);
	m_pEnableServer->EnableWindow(true);

	m_bServerEnabled = false;
	CountActiveConnections();	// update control
	CountActiveGames();			// update control
}

IN_ADDR CCaseStudyGame_Server_DSDlg::GetLocalHostAddressDetails()
{	
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		MessageBox("Failed to get local address!","Server");
		this->OnCancel();
	}
	return addr;
}

void CCaseStudyGame_Server_DSDlg::InitialiseLocalAddressStructAndPort()
{	// Initialise and display the IP address and Port number
	// The port value can be overwritten by the user, hence the 
	// subsequent use of ReadFinalPortValue() to read the final value
	m_LocalSockAddr.sin_addr = GetLocalHostAddressDetails();
	m_LocalSockAddr.sin_family = AF_INET;
	
	CString csStr;
	csStr.Format("%d",PORT);
	m_pListenPort->SetWindowText(csStr.GetString());
	m_LocalSockAddr.sin_port = htons((u_short) PORT);
	
	// Display the local address value
	csStr.Format("%d.%d.%d.%d", 
		m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b1,
        m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b2, 
		m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b3, 
		m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b4);
	m_pLocalAddress->SetWindowText(csStr.GetString());
}

void CCaseStudyGame_Server_DSDlg::ReadFinalPortValue()
{	
	// The Port number can be overwritten by the user since it was
	// automatically set to the default value. Hence the need to read the final value
	CString csStr;
	m_pListenPort->GetWindowText(csStr);
	u_short iPort = (u_short) atol(csStr.GetString());
	m_LocalSockAddr.sin_port = htons(iPort);
}

void CCaseStudyGame_Server_DSDlg::Accept()
{
	CString csStr;
	int iIndex = GetIndexOfUnusedConnection();
	if(iIndex == -1)
	{
		csStr = "Accept refused - Maximum number of Connections reached";
		WriteStatusLine(csStr);
		return;
	}
	
	int iRemoteAddrLen = sizeof(SOCKADDR_IN);
	int iError;
	bool bResult = false;
	
	m_ConnectionArray[iIndex].m_iConnectSock = accept(m_iListenSock, (SOCKADDR FAR*)&m_ConnectionArray[iIndex].m_ConnectSockAddr, &iRemoteAddrLen);
	if(INVALID_SOCKET == m_ConnectionArray[iIndex].m_iConnectSock)
	{
		iError = WSAGetLastError();
		// If no connection request is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	
			MessageBox("Accept failed","CNoughtsCrossesSERVER");
		}
	}
	else
	{
		// Set the newly-created connection socket to Non-Blocking
		u_long lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
		iError = ioctlsocket(m_ConnectionArray[iIndex].m_iConnectSock, FIONBIO, &lBlocking); // Set non-blocking socket behaviour
		if(NO_ERROR != iError)
		{
			MessageBox("Could not set non-Blocking behaviour for connection socket","CNoughtsCrossesSERVER");
			return;	
		}

		m_ConnectionArray[iIndex].bInUse = true;
		csStr.Format("Client %d.%d.%d.%d Connected", 
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1,
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2, 
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3,
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
		WriteStatusLine(csStr);
		CountActiveConnections(); // updates control
	}
}

void CCaseStudyGame_Server_DSDlg::OnBnClickedEnableServer()
{
	// 1. Create Listen Socket and set it NonBlocking
	m_iListenSock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC); // Create the socket
	if(INVALID_SOCKET == m_iListenSock)
	{
		MessageBox("Socket create failed","CNoughtsCrossesSERVER");
		return;	
	}
	
	// 2. Set non-blocking socket behaviour
	u_long lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	int iError = ioctlsocket(m_iListenSock, FIONBIO, &lBlocking); 
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set non-Blocking behaviour for Listen socket","CNoughtsCrossesSERVER");
		return;	
	}

	// 3. Read the port value because the user may have changed it from the default
	ReadFinalPortValue();

	// 4. Bind
	int iResult = bind(m_iListenSock, (const SOCKADDR FAR*)&m_LocalSockAddr, sizeof(m_LocalSockAddr));
	if(SOCKET_ERROR == iResult)
	{
		MessageBox("Bind failed !\nPort is probably in use","CNoughtsCrossesSERVER");
		return;
	}
	else
	{
		m_pListenPort->EnableWindow(false);
	}

	// 5. Listen - for TCP connection requests from clients
	iResult = listen(m_iListenSock, SOMAXCONN); 
	if(SOCKET_ERROR == iResult)
	{
		MessageBox("Listen failed","CNoughtsCrossesSERVER");
		return;
	}
	else
	{
		m_bServerEnabled = true;
		m_pEnableServer->EnableWindow(false);
		SendRegisterMessage();		// Auto-register with Directory service immediately
		InitiateRegistrationTimer();// Auto-register with Directory service periodically
		MainLoop();
	}
}

int CCaseStudyGame_Server_DSDlg::GetIndexOfUnusedConnection()
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == false)
		{
			return i;
		}
	}
	return -1;	// Signal that all connections are in use
}

int CCaseStudyGame_Server_DSDlg::CountActiveConnections()
{
	int iCount = 0;
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			iCount++;;
		}
	}
	// update number of connections control
	CString csStr;
	csStr.Format("%d",iCount);
	m_ConnectedCount.SetWindowText(csStr.GetString());
	return iCount;
}

int CCaseStudyGame_Server_DSDlg::CountActiveGames()
{
	int iCount = 0;
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == true)
		{
			iCount++;;
		}
	}
	// update number of connections control
	CString csStr;
	csStr.Format("%d",iCount);
	m_ActiveGamesCount.SetWindowText(csStr.GetString());
	return iCount;
}

int CCaseStudyGame_Server_DSDlg::FindIndexOfConnectionForAlias(CString csAlias)
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].csAlias.CompareNoCase(csAlias) == 0)
		{
			return i;
		}
	}
	return -1;	// Signal that Alias name was not found in connection array
}

int CCaseStudyGame_Server_DSDlg::GetIndexOfUnusedGame()
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == false)
		{
			return i;
		}
	}
	return -1;	// Signal that all connections are in use
}

bool CCaseStudyGame_Server_DSDlg::IsConnectionPartOfActiveGame(int iConnection)
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == true)
		{
			if(m_GameArray[i].iConnection[0] == iConnection	||
				m_GameArray[i].iConnection[1] == iConnection)
			{	
				return true;
			}
		}
	}
	return false;	// Signal connections is not part of an active game
}

int CCaseStudyGame_Server_DSDlg::GetConnectionOpponent(int iConnection)
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == true)
		{
			if(m_GameArray[i].iConnection[0] == iConnection)
			{
				return m_GameArray[i].iConnection[1];
			}	
			if(m_GameArray[i].iConnection[1] == iConnection)
			{
				return m_GameArray[i].iConnection[0];
			}
		}
	}
    return -1;
}

void CCaseStudyGame_Server_DSDlg::MainLoop()	// All Accept() and Receive() activity managed here
{
	Accept();
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			DoReceive(i);	// For each valid listen-socket DoReceive()
		}
	}
	InitiateTimer();   // As all activity is non-blocking
}

void CCaseStudyGame_Server_DSDlg::InitiateTimer()
{	
	m_nTimer = SetTimer(1, 100, 0);	// Short interval to allow UI responsiveness
}

void CCaseStudyGame_Server_DSDlg::InitiateRegistrationTimer()
{	
	m_nTimer = SetTimer(2, 10000, 0);	// 10 second interval between auto-registration activity
}

void CCaseStudyGame_Server_DSDlg::StopTimer()
{
	KillTimer(m_nTimer);
}

void CCaseStudyGame_Server_DSDlg::OnTimer(UINT nIDEvent) 
{	
	if(nIDEvent == 1)
	{
		StopTimer();
		MainLoop();
	}
	if(nIDEvent == 2) // Registration interval has expired
	{
		SendRegisterMessage();	// Periodic auto-registration
	}
}

void CCaseStudyGame_Server_DSDlg::Send(int iConnection, Message_PDU Message)
{
	int iError;
	int iBytesSent = send(m_ConnectionArray[iConnection].m_iConnectSock,
					(char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesSent)
	{
		iError = WSAGetLastError();
		ConnectionLost(iConnection, iError);
	}
}
				
void CCaseStudyGame_Server_DSDlg::SendPlayerList()
{
	Message_PDU Message;
	Message.iMessageType = PLAYER_LIST;
	strcpy(Message.cPlayerList,"");
	
	// List should include all Aliases that are connected, but not involved in active games
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true && IsConnectionPartOfActiveGame(i) == false &&
			m_ConnectionArray[i].csAlias.IsEmpty() == false)
		{
			strcat(Message.cPlayerList,m_ConnectionArray[i].csAlias.GetString());
			strcat(Message.cPlayerList,","); // Needs to be a comma-separated list
		}
	}
	if(strlen(Message.cPlayerList) > 0)
	{
		Message.cPlayerList[strlen(Message.cPlayerList) - 1] = 0; // Null terminate (write over last comma)
	}
	// Send PlayerList to ALL connected players
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			Send(i, Message);
		}
	}
}

void CCaseStudyGame_Server_DSDlg::SendOpponentCellSelectionMessage(int iConnection, int iCell)
{
	CString csStr;
	int iOpponentConnection;
	iOpponentConnection = GetConnectionOpponent(iConnection);	// find players opponent
	if(iOpponentConnection < 0)
	{
		csStr.Format("Opponent not found for %d",iConnection);
		WriteStatusLine(csStr);
		return;
	}
	Message_PDU Message;
	Message.iMessageType = OPPONENT_CELL_SELECTION;
	Message.iCell = iCell;
	Send(iOpponentConnection, Message);
}

void CCaseStudyGame_Server_DSDlg::DoReceive(int iConnection)
{		
	Message_PDU Message;
	int iBytesRecd = recv(m_ConnectionArray[iConnection].m_iConnectSock, (char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesRecd)
	{
		int iError = WSAGetLastError();
		// If no packet is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	
			ConnectionLost(iConnection, iError);	
			return;
		}
	}
	else
	{
		if(iBytesRecd == 0)
		{
			return;
		}

		CString csStr;
		switch(Message.iMessageType)
		{
		  case REGISTER_ALIAS:
			csStr.Format("Alias '%s' Registered", Message.cAlias);
			WriteStatusLine(csStr);
			m_ConnectionArray[iConnection].csAlias.Format("%s", Message.cAlias);
			SendPlayerList();
			break;
		  case CHOOSE_OPPONENT:
			if(CreateGame(iConnection, Message) == false)
			{	
				csStr.Format("ChooseOpponent '%s' FAILED", Message.cAlias);
			}
			WriteStatusLine(csStr);
			break;
		  case LOCAL_CELL_SELECTION:
			SendOpponentCellSelectionMessage(iConnection, Message.iCell);
			SetCellInGame(iConnection, Message.iCell);
			if(!CheckForWin(iConnection))
			{
				EndOfGame(iConnection);	// Check for end of game (all cells full)
			}
			break;
		  case CLIENT_DISCONNECT:
			csStr.Format("Client %s Disconnected", m_ConnectionArray[iConnection].csAlias.GetString());
			WriteStatusLine(csStr);
			CloseGameAndConnection(iConnection);
			break;
		}
	}
}

bool CCaseStudyGame_Server_DSDlg::CreateGame(int iConnection, Message_PDU Message)
{
	// Check initiating connection is not part of a game
	if(IsConnectionPartOfActiveGame(iConnection) == true)
	{
		return false;
	}
	// Check chosen opponent alias is connected
	CString csAlias(Message.cAlias);
	int iOpponentConnection;
	iOpponentConnection = FindIndexOfConnectionForAlias(csAlias);
	if(iOpponentConnection == -1)
	{
		return false;
	}
	// Check chosen opponent alias connection is not part of a game
	if(IsConnectionPartOfActiveGame(iOpponentConnection) == true)
	{
		return false;
	}
	// Get next available game structure in array
	int iGame;
	iGame = GetIndexOfUnusedGame();
	if(iGame == -1)
	{
		return false;
	}

	m_GameArray[iGame].iConnection[0] = iConnection;
	m_GameArray[iGame].iConnection[1] = iOpponentConnection;
	m_GameArray[iGame].bInUse = true;
	CountActiveGames(); // updates control
	bool bFlag = CreateRandomBool(); // Determine who goes first
	
	// Initialise game Grid
	for(int j = 0; j < 3; j++)
	{	
		for(int k = 0; k < 3; k++)
		{	
			m_GameArray[iGame].iGrid[j][k] = -1; // -1 = cell unoccupied (0 = '0' 1 = 'X')
		}	
	}
			
	// Send updated list of available players
	SendPlayerList();

	// Send START_OF_GAME messages to both players
	Message_PDU SendMessage;
	SendMessage.iMessageType = START_OF_GAME;
	strncpy(SendMessage.cAlias,m_ConnectionArray[iOpponentConnection].csAlias.GetString(),MAX_ALIAS_LEN);
	SendMessage.iToken = GetPlayerToken(iGame, iConnection);	
	SendMessage.bFirstMove = bFlag;
	Send(iConnection, SendMessage);
	strncpy(SendMessage.cAlias,m_ConnectionArray[iConnection].csAlias.GetString(),MAX_ALIAS_LEN);
	SendMessage.iToken = GetPlayerToken(iGame, iOpponentConnection);	
	SendMessage.bFirstMove = !bFlag;
	Send(iOpponentConnection, SendMessage);

	CString csStr;
	csStr.Format("Game '%s-%s' created", 
		m_ConnectionArray[m_GameArray[iGame].iConnection[0]].csAlias.GetString(),
		m_ConnectionArray[m_GameArray[iGame].iConnection[1]].csAlias.GetString());
	WriteStatusLine(csStr);
	return true;
}

int CCaseStudyGame_Server_DSDlg::FindGameForConnection(int iConnection)			
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].iConnection[0] == iConnection || 
			m_GameArray[i].iConnection[1] == iConnection)
		{
			return i;
		}
	}
	return -1;
}

int CCaseStudyGame_Server_DSDlg::GetPlayerToken(int iGame, int iConnection)			
{
	// Lowest numbered connection is '0'
	if(iGame >= 0 && iGame < MAX_GAMES)
	{
		if(	(m_GameArray[iGame].iConnection[0] == iConnection && 
				m_GameArray[iGame].iConnection[0] < m_GameArray[iGame].iConnection[1])
			||
			(m_GameArray[iGame].iConnection[1] == iConnection && 
				m_GameArray[iGame].iConnection[1] < m_GameArray[iGame].iConnection[0]) )
		{
			return 0;	// player identified by iConnection is '0'
		}
		else
		{
			return 1;	// player identified by iConnection is '1'
		}
	}
	return -1;	// iGame out of range
}

void CCaseStudyGame_Server_DSDlg::DoSetCellInGame(int iGame, int iCell, int iToken)			
{
	if((iToken == 0 || iToken == 1) && (iGame >= 0 && iGame < MAX_GAMES) 
		&& (iCell >= 0 && iCell < 9))
	{
		// Cell 0 [0][0], 1 [0][1], 2 [0][2]
		//      3 [1][0], 4 [1][1], 5 [1][2]
		//		6 [2][0], 7 [2][1], 8 [2][2]
		m_GameArray[iGame].iGrid[iCell / 3][iCell % 3] = iToken;
	}
	else
	{
		CString csStr;
		csStr.Format("Could not set cell %d, in game %d", iCell, iGame);
		WriteStatusLine(csStr);
    }
}

void CCaseStudyGame_Server_DSDlg::SetCellInGame(int iConnection, int iCell)			
{
	CString csStr;
	int iGame;
	iGame  = FindGameForConnection(iConnection);
	if(iGame  == -1)
	{	
		csStr.Format("Game for connection %d not found", iConnection);
		WriteStatusLine(csStr);
		return;
	}

	// Check if player is '0' or 'X' ('0' is always assigned to the lowest numbered connection)
	int iToken;
	iToken = GetPlayerToken(iGame, iConnection);
	if(iToken  == -1)
	{	
		csStr.Format("Token for connection %d (game %d) not found", iConnection, iGame);
		WriteStatusLine(csStr);
		return;
	}

	DoSetCellInGame(iGame, iCell, iToken);	
}
	
bool CCaseStudyGame_Server_DSDlg::CheckForLine(int iGame, int iToken)
{
	// check for win on diagonals
	if(m_GameArray[iGame].iGrid[1][1] == iToken &&
		( (m_GameArray[iGame].iGrid[0][0] == iToken &&
			m_GameArray[iGame].iGrid[2][2] == iToken)
			||
		  (m_GameArray[iGame].iGrid[0][2] == iToken &&
			m_GameArray[iGame].iGrid[2][0] == iToken) ) )
	{
		return true;
	}
	// check for horizontal line win
	for(int j = 0; j < 3; j++)
	{
		if(m_GameArray[iGame].iGrid[j][0] == iToken &&
			m_GameArray[iGame].iGrid[j][1] == iToken &&
			m_GameArray[iGame].iGrid[j][2] == iToken)
		{
			return true;
		}
	}
	// check for vertical line win
	for(int j = 0; j < 3; j++)
	{
		if(m_GameArray[iGame].iGrid[0][j] == iToken &&
			m_GameArray[iGame].iGrid[1][j] == iToken &&
			m_GameArray[iGame].iGrid[2][j] == iToken)
		{
			return true;
		}
	}
	return false;
}

bool CCaseStudyGame_Server_DSDlg::CheckForWin(int iConnection)
{
	// Player iConnection has just made a move
	// Thus only that player could have won
	int iGame;
	iGame = FindGameForConnection(iConnection);
	int iWinningToken;
	iWinningToken = GetPlayerToken(iGame, iConnection);
	// Seach for a line through the grid
	bool bResult;
	bResult = CheckForLine(iGame, iWinningToken);
	if(bResult == true)
	{
		Message_PDU Message;
		Message.iMessageType = END_OF_GAME;
		Message.iCode = END_OF_GAME_WIN;
		Send(iConnection, Message);
		int iOpponentConnection;
		iOpponentConnection = GetConnectionOpponent(iConnection);
		Message.iCode = END_OF_GAME_LOOSE;
		Send(iOpponentConnection, Message);
		RemoveGameAndConnections(iGame);
	}
	return bResult;
}

bool CCaseStudyGame_Server_DSDlg::EndOfGame(int iConnection)
{
	// if no cells free, signal end of game (Draw) to both players
	int iGame;
	iGame = FindGameForConnection(iConnection);
	
	for(int j = 0; j < 3; j++)
	{	
		for(int k = 0; k < 3; k++)
		{	
			if(m_GameArray[iGame].iGrid[j][k] == -1) // -1 = cell unoccupied (0 = '0' 1 = 'X')
			{
				return false;	// at least one unoccupied cell
			}
		}	
	}
	Message_PDU Message;
	Message.iMessageType = END_OF_GAME;
	Message.iCode = END_OF_GAME_DRAW;
	Send(iConnection, Message);
	int iOpponentConnection;
	iOpponentConnection = GetConnectionOpponent(iConnection);
	Send(iOpponentConnection, Message);
	RemoveGameAndConnections(iGame);
	return true;
}

void CCaseStudyGame_Server_DSDlg::RemoveGameAndConnections(int iGame)
{
	CloseConnection(m_GameArray[iGame].iConnection[0]);
	m_ConnectionArray[m_GameArray[iGame].iConnection[0]].bInUse = false;
	CloseConnection(m_GameArray[iGame].iConnection[1]);
	m_ConnectionArray[m_GameArray[iGame].iConnection[1]].bInUse = false;
	m_GameArray[iGame].bInUse = false;
	CountActiveGames(); // updates control
}

void CCaseStudyGame_Server_DSDlg::ConnectionLost(int iConnection, int iError)			
{
	if(WSAECONNABORTED == iError || WSAECONNRESET == iError ||
		WSAETIMEDOUT == iError || WSAENETRESET == iError ||
		WSAESHUTDOWN == iError)
	{
		CloseGameAndConnection(iConnection);
	}
}

void CCaseStudyGame_Server_DSDlg::CloseGameAndConnection(int iConnection)
{	
	int iGame = FindGameForConnection(iConnection);
	if(iGame != -1)
	{
		RemoveGameAndConnections(iGame);
	}
	else
	{
		CloseConnection(iConnection);
	}
}

void CCaseStudyGame_Server_DSDlg::CloseConnection(int iConnection)
{	
	shutdown(m_ConnectionArray[iConnection].m_iConnectSock,SD_BOTH);	// Close the connection
	closesocket(m_ConnectionArray[iConnection].m_iConnectSock);			// Close the socket
	m_ConnectionArray[iConnection].bInUse = false;
	m_ConnectionArray[iConnection].csAlias = "";
	CountActiveConnections();
	SendPlayerList();
		
	CString csStr;
	csStr.Format("%d.%d.%d.%d Disconnected", 
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1,
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2, 
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3,
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
	WriteStatusLine(csStr);
}

void CCaseStudyGame_Server_DSDlg::CloseAll()
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)	// Close active connections
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			CloseConnection(i);
		}
	}

	closesocket(m_iListenSock);		// Close the listen socket
	closesocket(m_RegisterSock);	// Close the registration socket
}

void CCaseStudyGame_Server_DSDlg::OnBnClickedClose()
{
	CloseAll();
	this->OnCancel();
}

void CCaseStudyGame_Server_DSDlg::WriteStatusLine(CString csLine)
{
	CString csExistingStatusText;
	m_pStatus->GetWindowText(csExistingStatusText);
	if(csExistingStatusText.GetLength())	// Create a new line if lines already exist
	{
		csExistingStatusText += "\r\n";
	}
	csExistingStatusText += csLine;
	m_pStatus->SetWindowText(csExistingStatusText.GetString());
}

void CCaseStudyGame_Server_DSDlg::SeedRandomNumberGenerator()
{
	__time64_t ltime;
    _time64(&ltime);	// Use the number of Seconds since 1/1/1970 as a seed
	srand((unsigned int)ltime );
}

bool CCaseStudyGame_Server_DSDlg::CreateRandomBool()
{
	int iRand = rand()%(1000);
	if(iRand >= 500)
	{
        return true;
	}
	return false;
}

void CCaseStudyGame_Server_DSDlg::OnBnClickedClearEventList()
{
	m_pStatus->SetWindowText("");
}

bool CCaseStudyGame_Server_DSDlg::CreateRegisterSocket()
{   // UDP, non-blocking, broadcast
	m_RegisterSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_RegisterSock)
	{
		return false;	// Error creating socket
	}

	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(m_RegisterSock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting non-blocking behaviour
	}

	// Turn on Broadcast mode
	char cOpt[2];
	cOpt[0] = 1; // true
	cOpt[1] = 0; // null terminate the option array
	iError = setsockopt(m_RegisterSock, SOL_SOCKET, SO_BROADCAST, cOpt, sizeof(cOpt)); 
  	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting broadcast option
	}
	return true;
}

void CCaseStudyGame_Server_DSDlg::SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, unsigned short port)
{	
	Sockaddr_In.sin_addr = GetBroadcastAddress();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

IN_ADDR CCaseStudyGame_Server_DSDlg::GetBroadcastAddress()
{	
	IN_ADDR addr;
	addr.S_un.S_un_b.s_b1 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b2 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b3 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b4 = (unsigned  char) 255;
	return addr;
}

void CCaseStudyGame_Server_DSDlg::SendRegisterMessage() // Broadcast message to Directory Server
{
	CString csStr;
	char cBuffer[50];
	char cPort[6];
	_itoa_s(PORT, cPort, 10);  // Supply the port that the game uses, so the Directory Service can inform clients
	strcpy_s(cBuffer,"REGISTER:");
	strcat_s(cBuffer,m_csAppName.GetString());
	strcat_s(cBuffer,":");
	strcat_s(cBuffer,cPort);
	int iBytesSent;

	iBytesSent = sendto(m_RegisterSock, (char *) cBuffer, sizeof(cBuffer), 0,
							(const sockaddr*)&m_RegisterAddr, sizeof(sockaddr));
	if(SOCKET_ERROR == iBytesSent)
	{
		csStr.Format("sendto (Directory Service) failed");
		WriteStatusLine(csStr);
	}
	if(false == bRegisterAlreadyDone)
	{
		csStr.Format("Registered with Directory Service");
		WriteStatusLine(csStr);
		bRegisterAlreadyDone = true;
	}
}
